@@ -31,12 +31,16 @@
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 
-#include "yelp-uri.h"
 #include "yelp-error.h"
-#include "yelp-theme.h"
+#include "yelp-settings.h"
 #include "yelp-toc-pager.h"
+#include "yelp-utils.h"
 
+#ifdef YELP_DEBUG
+#define d(x) x
+#else
 #define d(x)
+#endif
 
 typedef gboolean      (*ProcessFunction)        (YelpTocPager      *pager);
 
@@ -46,7 +50,7 @@ typedef struct _YelpMetafile YelpMetafile;
 typedef struct _YelpMenu     YelpMenu;
 
 struct _YelpTocPagerPriv {
-    GSList       *omf_dir_pending;
+    gboolean      omf_cl_docomf;
     GSList       *omf_pending;
 
     GSList       *man_dir_pending;
@@ -121,15 +125,19 @@ static void          toc_pager_class_init      (YelpTocPagerClass *klass);
 static void          toc_pager_init            (YelpTocPager      *pager);
 static void          toc_pager_dispose         (GObject           *gobject);
 
+static void          toc_pager_error           (YelpPager        *pager);
+static void          toc_pager_cancel          (YelpPager        *pager);
+static void          toc_pager_finish          (YelpPager        *pager);
+
 gboolean             toc_pager_process         (YelpPager         *pager);
 void                 toc_pager_cancel          (YelpPager         *pager);
-gchar *              toc_pager_resolve_uri     (YelpPager         *pager,
-						YelpURI           *uri);
-const GtkTreeModel * toc_pager_get_sections    (YelpPager         *pager);
+const gchar *        toc_pager_resolve_frag    (YelpPager         *pager,
+						const gchar       *frag_id);
+GtkTreeModel *       toc_pager_get_sections    (YelpPager         *pager);
 
 static gboolean      toc_process_pending       (YelpTocPager      *pager);
 
-static gboolean      process_omf_dir_pending   (YelpTocPager      *pager);
+static gboolean      process_omf_cl            (YelpTocPager      *pager);
 static gboolean      process_omf_pending       (YelpTocPager      *pager);
 static gboolean      process_man_dir_pending   (YelpTocPager      *pager);
 static gboolean      process_man_pending       (YelpTocPager      *pager);
@@ -194,9 +202,13 @@ toc_pager_class_init (YelpTocPagerClass *klass)
 
     object_class->dispose = toc_pager_dispose;
 
+    pager_class->error        = toc_pager_error;
+    pager_class->cancel       = toc_pager_cancel;
+    pager_class->finish       = toc_pager_finish;
+
     pager_class->process      = toc_pager_process;
     pager_class->cancel       = toc_pager_cancel;
-    pager_class->resolve_uri  = toc_pager_resolve_uri;
+    pager_class->resolve_frag = toc_pager_resolve_frag;
     pager_class->get_sections = toc_pager_get_sections;
 }
 
@@ -234,12 +246,13 @@ toc_pager_dispose (GObject *object)
 void
 yelp_toc_pager_init (void)
 {
-    YelpURI *uri;
+    YelpDocInfo *doc_info;
 
-    uri = yelp_uri_new ("toc:");
+    doc_info = yelp_doc_info_get ("x-yelp-toc:");
     
-    toc_pager = (YelpTocPager *) g_object_new (YELP_TYPE_TOC_PAGER, 
-    					       "uri", uri, NULL);
+    toc_pager = (YelpTocPager *) g_object_new (YELP_TYPE_TOC_PAGER,
+					       "document-info", doc_info,
+					       NULL);
 
     yelp_pager_start (YELP_PAGER (toc_pager));
 }
@@ -254,6 +267,10 @@ void
 yelp_toc_pager_pause (YelpTocPager *pager)
 {
     g_return_if_fail (pager != NULL);
+
+    d (g_print ("yelp_toc_pager_pause\n"));
+    d (g_print ("  pause_caunt = %i\n", pager->priv->pause_count + 1));
+
     pager->priv->pause_count = pager->priv->pause_count + 1;
 }
 
@@ -261,8 +278,11 @@ void
 yelp_toc_pager_unpause (YelpTocPager *pager)
 {
     g_return_if_fail (pager != NULL);
-    pager->priv->pause_count = pager->priv->pause_count - 1;
 
+    d (g_print ("yelp_toc_pager_unpause\n"));
+    d (g_print ("  pause_caunt = %i\n", pager->priv->pause_count - 1));
+
+    pager->priv->pause_count = pager->priv->pause_count - 1;
     if (pager->priv->pause_count < 0) {
 	g_warning (_("YelpTocPager: Pause count is negative."));
 	pager->priv->pause_count = 0;
@@ -277,15 +297,41 @@ yelp_toc_pager_unpause (YelpTocPager *pager)
 
 /******************************************************************************/
 
+static void
+toc_pager_error (YelpPager *pager)
+{
+    d (g_print ("toc_pager_error\n"));
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_ERROR);
+}
+
+static void
+toc_pager_cancel (YelpPager *pager)
+{
+    YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
+
+    d (g_print ("toc_pager_cancel\n"));
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_INVALID);
+
+    priv->cancel = TRUE;
+}
+
+static void
+toc_pager_finish (YelpPager   *pager)
+{
+    d (g_print ("toc_pager_finish\n"));
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
+}
+
 gboolean
 toc_pager_process (YelpPager *pager)
 {
     gchar  *manpath;
     YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
 
-    /* Set the OMF directories to be read */
-    priv->omf_dir_pending = g_slist_prepend (priv->omf_dir_pending,
-					     g_strdup (DATADIR "/omf"));
+    d (g_print ("toc_pager_process\n"));
+
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_PARSING);
+    g_signal_emit_by_name (pager, "parse");
 
     /* Set the man directories to be read */
     if (!g_spawn_command_line_sync ("manpath", &manpath, NULL, NULL, NULL))
@@ -309,7 +355,10 @@ toc_pager_process (YelpPager *pager)
     /* Set the info directories to be read */
 
     /* Set it running */
-    priv->pending_func = (ProcessFunction) process_omf_dir_pending;
+    yelp_pager_set_state (pager, YELP_PAGER_STATE_RUNNING);
+    g_signal_emit_by_name (pager, "start");
+
+    priv->pending_func = (ProcessFunction) process_omf_cl;
 
     gtk_idle_add_priority (G_PRIORITY_LOW,
 			   (GtkFunction) toc_process_pending,
@@ -317,26 +366,16 @@ toc_pager_process (YelpPager *pager)
     return FALSE;
 }
 
-void
-toc_pager_cancel (YelpPager *pager)
+const gchar *
+toc_pager_resolve_frag (YelpPager *pager, const gchar *frag_id)
 {
-    YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
-
-    priv->cancel = TRUE;
-}
-
-gchar *
-toc_pager_resolve_uri (YelpPager *pager, YelpURI *uri)
-{
-    const gchar *frag = gnome_vfs_uri_get_fragment_identifier (uri->uri);
-
-    if (!frag)
-	return g_strdup ("index");
+    if (!frag_id)
+	return "index";
     else
-	return g_strdup (frag);
+	return frag_id;
 }
 
-const GtkTreeModel *
+GtkTreeModel *
 toc_pager_get_sections (YelpPager *pager)
 {
     return NULL;
@@ -358,7 +397,7 @@ toc_process_pending (YelpTocPager *pager)
     readd = priv->pending_func(pager);
 
     if (!readd) {
-	if (priv->pending_func == process_omf_dir_pending)
+	if (priv->pending_func == process_omf_cl)
 	    priv->pending_func = process_omf_pending;
 	else if (priv->pending_func == process_omf_pending)
 	    priv->pending_func = process_man_dir_pending;
@@ -384,72 +423,84 @@ toc_process_pending (YelpTocPager *pager)
     if (priv->pending_func == NULL)
 	g_signal_emit_by_name (pager, "finish");
 
-    if (priv->pending_func && (priv->pause_count < 1))
+    if ((priv->pending_func != NULL) && (priv->pause_count < 1))
 	return TRUE;
     else
 	return FALSE;
 }
 
+/** process_omf_cl ************************************************************/
+
+static void
+omf_cl_startElement (void           *pager,
+		     const xmlChar  *name,
+		     const xmlChar **attrs)
+{
+    YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
+    if (xmlStrEqual((const xmlChar*) name, "docomf"))
+	priv->omf_cl_docomf = TRUE;
+}
+
+static void
+omf_cl_endElement (void          *pager,
+		   const xmlChar *name)
+{
+    YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
+    if (xmlStrEqual((const xmlChar*) name, "docomf"))
+	priv->omf_cl_docomf = FALSE;
+}
+
+static void
+omf_cl_characters (void          *pager,
+		   const xmlChar *ch,
+		   int            len)
+{
+    gchar *omf;
+    YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
+
+    if (priv->omf_cl_docomf) {
+	omf = g_strndup (ch, len);
+	priv->omf_pending = g_slist_prepend (priv->omf_pending, omf);
+    }
+}
+
 static gboolean
-process_omf_dir_pending (YelpTocPager *pager)
+process_omf_cl (YelpTocPager *pager)
 {
-    GSList  *first  = NULL;
-    gchar   *dirstr = NULL;
-    GnomeVFSResult           result;
-    GnomeVFSDirectoryHandle *dir       = NULL;
-    GnomeVFSFileInfo        *file_info = NULL;
+    gchar  *content_list;
+    gchar  *stderr;
+    GList  *langs;
+    gchar  *lang;
+    gchar  *command;
+    static xmlSAXHandler omf_cl_sax_handler = { 0, };
 
-    YelpTocPagerPriv *priv = pager->priv;
+    langs = (GList *) gnome_i18n_get_language_list (NULL);
+    if (langs && langs->data)
+	lang = (gchar *) langs->data;
+    else
+	lang = "C";
 
-    first = priv->omf_dir_pending;
-    priv->omf_dir_pending = g_slist_remove_link (priv->omf_dir_pending, first);
-    if (!first)
-	goto done;
+    command = g_strconcat("scrollkeeper-get-content-list ", lang, NULL);
 
-    dirstr = (gchar *) first->data;
-    if (!dirstr)
-	goto done;
-
-    result = gnome_vfs_directory_open (&dir, dirstr,
-				       GNOME_VFS_FILE_INFO_DEFAULT);
-    if (result != GNOME_VFS_OK)
-	goto done;
-
-    file_info = gnome_vfs_file_info_new ();
-
-    while (gnome_vfs_directory_read_next (dir, file_info) == GNOME_VFS_OK) {
-	switch (file_info->type) {
-	case GNOME_VFS_FILE_TYPE_DIRECTORY:
-	    if (strcmp (file_info->name, ".") && strcmp (file_info->name, "..")) {
-		gchar *newdir = g_strconcat (dirstr, "/", file_info->name, NULL);
-		priv->omf_dir_pending = g_slist_prepend (priv->omf_dir_pending, newdir);
-	    }
-	    break;
-	case GNOME_VFS_FILE_TYPE_REGULAR:
-	case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
-	    priv->omf_pending =
-		g_slist_prepend (priv->omf_pending,
-				 g_strconcat (dirstr, "/", file_info->name, NULL));
-	    break;
-	default:
-	    break;
+    if (g_spawn_command_line_sync (command, &content_list, &stderr, NULL, NULL)) {
+	if (!omf_cl_sax_handler.startElement) {
+	    omf_cl_sax_handler.startElement = omf_cl_startElement;
+	    omf_cl_sax_handler.endElement   = omf_cl_endElement;
+	    omf_cl_sax_handler.characters   = omf_cl_characters;
+	    omf_cl_sax_handler.initialized  = TRUE;
 	}
+	content_list = g_strstrip (content_list);
+	xmlSAXUserParseFile (&omf_cl_sax_handler, pager, content_list);
     }
 
- done:
-    if (file_info)
-    	gnome_vfs_file_info_unref (file_info);
-    if (dir && !result)
-	gnome_vfs_directory_close (dir);
-    g_slist_free_1 (first);
-    g_free (dirstr);
-
-    if (priv->omf_dir_pending)
-	return TRUE;
-    else
-	return FALSE;
+    g_free (content_list);
+    g_free (stderr);
+    g_free (command);
+    return FALSE;
 }
 
+/** process_omf_pending *******************************************************/
+
 static gboolean
 process_omf_pending (YelpTocPager *pager)
 {
@@ -546,7 +597,7 @@ process_omf_pending (YelpTocPager *pager)
 	     * list of preferred languages for the user.  Low numbers are best.
 	     */
 	    lang_priority = 0;
-	    langs = (GList *) gnome_i18n_get_language_list ("LC_MESSAGES");
+	    langs = (GList *) gnome_i18n_get_language_list (NULL);
 	    for ( ; langs != NULL; langs = langs->next) {
 		gchar *lang = langs->data;
 		lang_priority++;
@@ -580,7 +631,7 @@ process_omf_pending (YelpTocPager *pager)
 	goto done;
     }
 
-    omf->id = g_strconcat ("scrollkeeper/", (gchar *) omf->seriesid, NULL);
+    omf->id = g_strconcat ("scrollkeeper.", (gchar *) omf->seriesid, NULL);
 
     // If we have one with the same id, use the one with the lowest lang_priority.
 
@@ -736,6 +787,8 @@ process_menu_pending (YelpTocPager *pager)
 
     YelpTocPagerPriv *priv = pager->priv;
 
+    d (g_print ("process_menu_pending\n"));
+
     first = priv->menu_pending;
     priv->menu_pending = g_slist_remove_link (priv->menu_pending, first);
 
@@ -772,9 +825,9 @@ process_menu_pending (YelpTocPager *pager)
 
     if (menu->has_submenus || menu->metafiles) {
 	YelpPage *page = g_new0 (YelpPage, 1);
-	page->chunk    = menu_write_page (menu);
-	page->id    = menu->id;
-	page->title = menu->title;
+	page->page_id  = menu->id;
+	page->title    = menu->title;
+	page->contents = menu_write_page (menu);
 
 	yelp_pager_add_page (YELP_PAGER (pager), page);
 	g_signal_emit_by_name (pager, "page", menu->id);
@@ -861,7 +914,7 @@ menu_write_page (YelpMenu  *menu)
 	 "<head><meta http-equiv='Content-Type'"
 	 " content='text/html=; charset=utf-8'>"
 	 "<link rel='stylesheet' type='text/css' href='%s'></head>\n",
-	 yelp_theme_get_css_file ());
+	 yelp_settings_get_css_file ());
 
     g_string_append_printf (gstr, "<body><h1>%s</h1>\n", menu->title);
 
@@ -887,7 +940,7 @@ menu_write_menu (YelpMenu  *menu,
 	    YelpMenu   *cur_menu = (YelpMenu *) cur->_private;
 	    if (cur_menu && (cur_menu->has_submenus || cur_menu->metafiles))
 		g_string_append_printf (gstr,
-					"<li><a href='toc:%s'>%s</a></li>\n",
+					"<li><a href='x-yelp-toc:%s'>%s</a></li>\n",
 					cur_menu->id,
 					cur_menu->title);
 	}
@@ -919,7 +972,7 @@ node_get_title (xmlNodePtr node)
 	if (!xmlStrcmp (cur->name, (const xmlChar *) "title")) {
 	    gint pri = 0;
 	    xmlChar *tlang = xmlNodeGetLang (cur);
-	    GList *langs = (GList *) gnome_i18n_get_language_list ("LC_MESSAGES");
+	    GList *langs = (GList *) gnome_i18n_get_language_list (NULL);
 
 	    for ( ; langs != NULL; langs = langs->next) {
 		gchar *lang = langs->data;
